@@ -1,17 +1,15 @@
 # pa.py
 # Simon Hulse
 # simon.hulse@chem.ox.ac.uk
-# Last Edited: Wed 23 Mar 2022 17:46:29 GMT
+# Last Edited: Mon 09 May 2022 12:00:48 BST
 
 """Module for simulating 1D pulse-acquire experiments.
 
 **Pulse Sequence:**
 
 .. image:: ../figures/pa/pa.png
-
-The result of this pulse sequence is a pair of amplitude-modulated FIDs.
 """
-from typing import Tuple, Union
+from typing import Optional, Tuple, Union
 
 import numpy as np
 from numpy import fft
@@ -103,28 +101,83 @@ class PulseAcquireSimulation(Simulation):
             fid[i] = rho.expectation(detect) * recphase
             rho = rho.propagate(evol)
 
-        fid *= np.exp(np.linspace(0, -10, pts))
         return fid
 
-    def _fetch_fid(self) -> Tuple[np.ndarray, np.ndarray]:
+    def fid(
+        self,
+        lb: Optional[float] = None,
+    ) -> Tuple[np.ndarray, np.ndarray, str]:
+        """Return the FID associated with a simulation.
+
+        Parameters
+        ----------
+        lb
+            Line-broadening factor for exponential window function. Default
+            option (``None``), will apply an exponential window such that the final
+            point will be shrunk to 1/1000 of its original value.
+
+        Returns
+        -------
+        timepoints
+            The timepoints sampled.
+
+        fid
+            The FID sampled.
+
+        labels
+            Axis label for plotting purposes.
+        """
         pts, sw = self.points[0], self.sweep_widths[0]
         tp = np.linspace(0, (pts - 1) / sw, pts)
-        return tp, self._fid
 
-    def _fetch_spectrum(
-        self, zf_factor: int = 1, shift_frequency: str = "ppm"
+        if lb is None:
+            lb = -np.log(0.001) / (np.pi * (pts - 1))
+        em = np.exp(-np.pi * np.arange(pts) * lb)
+
+        return tp, self._fid * em, "$t$ (s)"
+
+    def spectrum(
+        self,
+        zf_factor: float = 1.0,
+        lb: float = None,
     ) -> Tuple[np.ndarray, np.ndarray]:
+        """Return the spectrum associated with a simulation.
+
+        Parameters
+        ----------
+        zf_factor
+            The ratio between the number of points in the final spectrum,
+            generated by zero-filling the FID, and the FID itself in each dimension.
+            ``1.0`` (default) means no zero-filling is applied. Should be ``>= 1.0``.
+
+        lb
+            Line-broadening factor for exponential window function. Default
+            option (``None``), will apply an exponential window such that the final
+            point will be shrunk to 1/1000 of its original value.
+
+        Returns
+        -------
+        shifts
+            The chemical shifts sampled.
+
+        spectrum
+            The spectrum generated from the FID.
+
+        labels
+            Axis label for plotting purposes.
+        """
         sw, off, pts, sfo = (
             self.sweep_widths[0], self.offsets[0], self.points[0], self.sfo[0],
         )
-        fid = self._fid
+        _, fid, _ = self.fid(lb=lb)
         fid[0] /= 2
-        shifts = np.linspace((sw / 2) + off, -(sw / 2) + off, pts * zf_factor) / sfo
+        shape = int(pts * zf_factor)
+        shifts = np.linspace((sw / 2) + off, -(sw / 2) + off, shape) / sfo
         spectrum = np.flip(
             fft.fftshift(
                 fft.fft(
                     fid,
-                    pts * zf_factor,
+                    shape,
                 )
             )
         )
@@ -155,4 +208,5 @@ if __name__ == "__main__":
     ax.plot(shifts, np.real(spectrum))
     ax.set_xlim(reversed(ax.get_xlim()))
     ax.set_xlabel(label)
+    ax.set_yticks([])
     plt.show()
